package com.loganmagnan.hubcore.managers;

import com.loganmagnan.hubcore.HubCore;
import com.loganmagnan.hubcore.managers.item.HotBarItem;
import com.loganmagnan.hubcore.playerdata.PlayerData;
import lombok.Getter;
import org.bukkit.entity.Player;
import org.bukkit.scheduler.BukkitRunnable;
import java.util.Collection;
import java.util.HashMap;
import java.util.Map;
import java.util.UUID;

public class PlayerDataManager {

	private HubCore main = HubCore.getInstance();

	@Getter private final Map<UUID, PlayerData> players = new HashMap<>();

	public PlayerData getOrCreate(UUID uniqueId) {
		return this.players.computeIfAbsent(uniqueId, PlayerData::new);
	}

	public PlayerData getPlayerData(UUID uniqueId) {
		return this.players.getOrDefault(uniqueId, new PlayerData(uniqueId));
	}

	public Collection<PlayerData> getAllPlayers() {
		return this.players.values();
	}

	public void loadPlayerData(PlayerData playerData) {
		playerData.setLoaded(true);
	}

	public void savePlayerData(PlayerData playerData) {

	}

	public void deletePlayer(UUID uniqueId) {
		this.main.getServer().getScheduler().runTaskAsynchronously(this.main, () -> {
			this.savePlayerData(this.players.get(uniqueId));
			this.players.remove(uniqueId);
		});
	}

	public void sendToSpawnAndResetPlayer(Player player) {
		player.getInventory().clear();

		this.giveSpawnItemsToPlayer(player);

		if (!player.isOnline()) {
			return;
		}

		this.resetPlayerView();

		player.teleport(this.main.getSpawnManager().getSpawnLocation().toBukkitLocation());
	}

	public void giveSpawnItemsToPlayer(Player player) {
		this.main.getHotBarManager().getSpawnItems().stream().filter(HotBarItem::isEnabled).forEach(item -> player.getInventory().setItem(item.getSlot(), item.getItemStack()));

		player.updateInventory();
	}

	public void resetPlayerView() {
		(new BukkitRunnable() {
			public void run() {
				PlayerDataManager.this.main.getServer().getOnlinePlayers().forEach(player -> {
					PlayerData playerData = PlayerDataManager.this.main.getPlayerDataManager().getPlayerData(player.getUniqueId());

					if (playerData.getPlayerSettings().isPlayerVisibilityEnabled()) {
						PlayerDataManager.this.main.getServer().getOnlinePlayers().forEach(playerOnline -> {
							player.showPlayer(playerOnline);
						});
					} else {
						PlayerDataManager.this.main.getServer().getOnlinePlayers().forEach(playerOnline -> {
							player.hidePlayer(playerOnline);
						});
					}
				});
			}
		}).runTaskAsynchronously(this.main);
	}
}
