package com.loganmagnan.hubcore.menus;

import com.loganmagnan.hubcore.HubCore;
import com.loganmagnan.hubcore.managers.cosmetic.Cosmetic;
import com.loganmagnan.hubcore.managers.cosmetic.CosmeticType;
import com.loganmagnan.hubcore.playerdata.PlayerData;
import com.loganmagnan.hubcore.playerdata.PlayerSettings;
import com.loganmagnan.hubcore.menusystem.PaginatedMenu;
import com.loganmagnan.hubcore.menusystem.PlayerMenuUtil;
import com.loganmagnan.hubcore.utils.ColorUtils;
import org.bukkit.ChatColor;
import org.bukkit.Material;
import org.bukkit.NamespacedKey;
import org.bukkit.entity.Player;
import org.bukkit.event.inventory.InventoryClickEvent;
import org.bukkit.inventory.ItemStack;
import org.bukkit.inventory.meta.ItemMeta;
import org.bukkit.persistence.PersistentDataType;

import java.util.ArrayList;
import java.util.List;

public class CosmeticTypeMenu extends PaginatedMenu {

    private HubCore main = HubCore.getInstance();

    private CosmeticType cosmeticType;

    public CosmeticTypeMenu(PlayerMenuUtil playerMenuUtil, CosmeticType cosmeticType) {
        super(playerMenuUtil);

        this.cosmeticType = cosmeticType;
    }

    @Override
    public String getMenuName() {
        return ColorUtils.getMessageType(this.main.getMenusConfig().getConfig().getString("MENUS." + this.cosmeticType.name() + ".TITLE"));
    }

    @Override
    public int getSlots() {
        return this.main.getMenusConfig().getConfig().getInt("MENUS." + this.cosmeticType.name() + ".SIZE");
    }

    @Override
    public void handleMenu(InventoryClickEvent event) {
        Player player = (Player) event.getWhoClicked();

        PlayerData playerData = this.main.getPlayerDataManager().getPlayerData(player.getUniqueId());

        PlayerSettings playerSettings = playerData.getPlayerSettings();

        if (event.getView().getTitle().equalsIgnoreCase(ColorUtils.getMessageType(this.main.getMenusConfig().getConfig().getString("MENUS." + this.cosmeticType.name() + ".TITLE")))) {
            switch (event.getCurrentItem().getType()) {
                case OAK_BUTTON:
                    if (page == 0) {
                        player.sendMessage(ColorUtils.getMessageType("&aYou are on the first page"));
                    } else {
                        page--;

                        super.open(player);
                    }

                    return;
                case STONE_BUTTON:
                    if (!((index + 1) >= this.main.getCosmeticManager().getCosmetics().size())) {
                        page++;

                        super.open(player);
                    } else {
                        player.sendMessage(ColorUtils.getMessageType("&aYou are on the last page"));
                    }

                    return;
                case BARRIER:
                    player.closeInventory();

                    return;
            }

            if (event.getCurrentItem().getType() != Material.AIR) {
                Cosmetic cosmetic = this.main.getCosmeticManager().getCosmetic(event.getCurrentItem().getItemMeta().getPersistentDataContainer().get(new NamespacedKey(this.main, "Effect"), PersistentDataType.STRING));

                if (cosmetic == null) {
                    return;
                }

                playerData.setEffect(cosmetic.getEffect());

                player.closeInventory();
                player.sendMessage(ColorUtils.getMessageType("&aSet your trail to &a&l" + ChatColor.stripColor(cosmetic.getItemStack().getItemMeta().getDisplayName())));

                return;
            }
        }
    }

    @Override
    public void setMenuItems(Player player) {
        this.addMenuBorder();

        for (int i = 0; i < this.getMaxItemsPerPage(); i++) {
            this.index = this.getMaxItemsPerPage() * this.page + i;

            if (this.index >= this.main.getCosmeticManager().getCosmetics().size()) {
                break;
            }

            List<Cosmetic> cosmetics = new ArrayList<Cosmetic>(this.main.getCosmeticManager().getCosmetics());

            for (Cosmetic cosmetic : cosmetics) {
                if (cosmetic.getCosmeticType() == this.cosmeticType) {
                    continue;
                }

                cosmetics.remove(cosmetic);
            }

            if (cosmetics.get(this.index) != null) {
                Cosmetic cosmetic = cosmetics.get(this.index);

                ItemStack cosmeticItemStack = cosmetic.getItemStack();

                ItemMeta itemMeta = cosmeticItemStack.getItemMeta();
                itemMeta.getPersistentDataContainer().set(new NamespacedKey(this.main, "Effect"), PersistentDataType.STRING, cosmetic.getEffect().name());

                cosmeticItemStack.setItemMeta(itemMeta);

                this.inventory.addItem(cosmeticItemStack);
            }
        }
    }
}
