package com.loganmagnan.hubcore.menus;

import com.loganmagnan.hubcore.HubCore;
import com.loganmagnan.hubcore.playerdata.PlayerData;
import com.loganmagnan.hubcore.playerdata.PlayerSettings;
import com.loganmagnan.hubcore.menusystem.ItemStackButton;
import com.loganmagnan.hubcore.menusystem.Menu;
import com.loganmagnan.hubcore.menusystem.PlayerMenuUtil;
import com.loganmagnan.hubcore.utils.ColorUtils;
import org.bukkit.Material;
import org.bukkit.entity.Player;
import org.bukkit.event.inventory.InventoryClickEvent;
import org.bukkit.inventory.ItemStack;

import java.util.ArrayList;
import java.util.List;

public class SettingsSelectorMenu extends Menu {

    private HubCore main = HubCore.getInstance();

    public SettingsSelectorMenu(PlayerMenuUtil playerMenuUtil) {
        super(playerMenuUtil);
    }

    @Override
    public String getMenuName() {
        return ColorUtils.getMessageType(this.main.getMenusConfig().getConfig().getString("MENUS.SETTINGS.TITLE"));
    }

    @Override
    public int getSlots() {
        return this.main.getMenusConfig().getConfig().getInt("MENUS.SETTINGS.SIZE");
    }

    @Override
    public void handleMenu(InventoryClickEvent event) {
        Player player = (Player) event.getWhoClicked();

        PlayerData playerData = this.main.getPlayerDataManager().getPlayerData(player.getUniqueId());

        PlayerSettings playerSettings = playerData.getPlayerSettings();

        if (event.getView().getTitle().equalsIgnoreCase(ColorUtils.getMessageType(this.main.getMenusConfig().getConfig().getString("MENUS.SETTINGS.TITLE")))) {
            switch (event.getCurrentItem().getType()) {
                case PAINTING:
                    playerSettings.setScoreboardEnabled(!playerSettings.isScoreboardEnabled());

                    break;
                case PLAYER_HEAD:
                    playerSettings.setPlayerVisibilityEnabled(!playerSettings.isPlayerVisibilityEnabled());

                    this.main.getPlayerDataManager().resetPlayerView();

                    break;
            }

            new SettingsSelectorMenu(this.playerMenuUtil).open(player);
        }
    }

    @Override
    public void setMenuItems(Player player) {
        PlayerData playerData = this.main.getPlayerDataManager().getPlayerData(player.getUniqueId());

        PlayerSettings playerSettings = playerData.getPlayerSettings();

        this.setFillerGlass();

        List<String> scoreboardLore = new ArrayList<String>(this.main.getMenusConfig().getConfig().getStringList("MENUS.SETTINGS.SETTINGS.SCOREBOARD.LORE"));
        scoreboardLore.add((playerSettings.isScoreboardEnabled() ? this.main.getMenusConfig().getConfig().getString("MENUS.SETTINGS.ENABLED") : this.main.getMenusConfig().getConfig().getString("MENUS.SETTINGS.UNSELECTED")) + " Scoreboard enabled");
        scoreboardLore.add((!playerSettings.isScoreboardEnabled() ? this.main.getMenusConfig().getConfig().getString("MENUS.SETTINGS.DISABLED") : this.main.getMenusConfig().getConfig().getString("MENUS.SETTINGS.UNSELECTED")) + " Scoreboard disabled");

        List<String> playerVisibilityLore = new ArrayList<String>(this.main.getMenusConfig().getConfig().getStringList("MENUS.SETTINGS.SETTINGS.PLAYER-VISIBILITY.LORE"));
        playerVisibilityLore.add((playerSettings.isPlayerVisibilityEnabled() ? this.main.getMenusConfig().getConfig().getString("MENUS.SETTINGS.ENABLED") : this.main.getMenusConfig().getConfig().getString("MENUS.SETTINGS.UNSELECTED")) + " Players enabled");
        playerVisibilityLore.add((!playerSettings.isPlayerVisibilityEnabled() ? this.main.getMenusConfig().getConfig().getString("MENUS.SETTINGS.DISABLED") : this.main.getMenusConfig().getConfig().getString("MENUS.SETTINGS.UNSELECTED")) + " Players disabled");

        ItemStackButton scoreboardItemStackButton = new ItemStackButton(
                this.main.getMenusConfig().getConfig().getString("MENUS.SETTINGS.SETTINGS.SCOREBOARD.NAME"),
                scoreboardLore,
                Material.getMaterial(this.main.getMenusConfig().getConfig().getString("MENUS.SETTINGS.SETTINGS.SCOREBOARD.MATERIAL")),
                0,
                1);

        ItemStackButton playerVisibilityItemStackButton = new ItemStackButton(
                this.main.getMenusConfig().getConfig().getString("MENUS.SETTINGS.SETTINGS.PLAYER-VISIBILITY.NAME"),
                playerVisibilityLore,
                Material.getMaterial(this.main.getMenusConfig().getConfig().getString("MENUS.SETTINGS.SETTINGS.PLAYER-VISIBILITY.MATERIAL")),
                0,
                1);

        ItemStack scoreboardItemStack = scoreboardItemStackButton.makeItemStack();
        ItemStack playerVisibilityItemStack = playerVisibilityItemStackButton.makeItemStack();

        this.inventory.setItem(this.main.getMenusConfig().getConfig().getInt("MENUS.SETTINGS.SETTINGS.SCOREBOARD.SLOT"), scoreboardItemStack);
        this.inventory.setItem(this.main.getMenusConfig().getConfig().getInt("MENUS.SETTINGS.SETTINGS.PLAYER-VISIBILITY.SLOT"), playerVisibilityItemStack);
    }
}
