package com.loganmagnan.hubcore.scoreboard;

import aether.scoreboard.Board;
import aether.scoreboard.BoardAdapter;
import aether.scoreboard.cooldown.BoardCooldown;
import com.loganmagnan.hubcore.HubCore;
import com.loganmagnan.hubcore.playerdata.PlayerData;
import com.loganmagnan.hubcore.playerdata.PlayerSettings;
import com.loganmagnan.hubcore.utils.ColorUtils;
import com.loganmagnan.hubcore.utils.Utils;
import com.loganmagnan.vortex.api.VortexAPI;
import org.bukkit.Bukkit;
import org.bukkit.ChatColor;
import org.bukkit.entity.Player;
import org.bukkit.scoreboard.Scoreboard;
import java.util.ArrayList;
import java.util.List;
import java.util.Set;
import java.util.regex.Matcher;
import java.util.regex.Pattern;

public class ScoreboardProvider implements BoardAdapter {

    private HubCore main = HubCore.getInstance();

    private Pattern HEX_PATTERN = Pattern.compile("&#([A-Fa-f0-9]{6})");

    @Override
    public String getTitle(Player player) {
        return ColorUtils.getMessageType(this.main.getScoreboardConfig().getConfig().getString("SCOREBOARD.TITLE"));
    }

    @Override
    public List<String> getScoreboard(Player player, Board board, Set<BoardCooldown> cooldowns) {
        PlayerData playerData = this.main.getPlayerDataManager().getPlayerData(player.getUniqueId());

        if (playerData == null) {
            return null;
        }

        PlayerSettings playerSettings = playerData.getPlayerSettings();

        if (!playerSettings.isScoreboardEnabled()) {
            return null;
        }

        switch (playerData.getPlayerState()) {
            case SPAWN:
                return this.getSpawnScoreboard(player, playerData);
        }

        return null;
    }

    @Override
    public void onScoreboardCreate(Player player, Scoreboard scoreboard) {

    }

    private List<String> getSpawnScoreboard(Player player, PlayerData playerData) {
        List<String> lines = new ArrayList<>();

        String rank = "";

        if (this.main.getServer().getPluginManager().getPlugin("Vault") == null) {
            if (this.main.getServer().getPluginManager().getPlugin("Vortex") == null) {
                rank = "None";
            } else {
                rank = VortexAPI.getRank(player).getPrefix();
            }
        } else {
            rank = this.main.getChat().getGroupPrefix(player.getWorld(), this.main.getChat().getPrimaryGroup(player));
        }

        for (String string : this.main.getScoreboardConfig().getConfig().getStringList("SCOREBOARD.LINES.SPAWN")) {
            lines.add(
                    string
                            .replace("%line%", Utils.scoreboardBar)
                            .replace("%online%", String.valueOf(this.main.getServer().getOnlinePlayers().size()))
                            .replace("%rank%", colorize(translateHexColorCodes(rank)))
            );
        }

        return lines;
    }

    public String colorize(String message) {
        return ChatColor.translateAlternateColorCodes('&', message);
    }

    public String translateHexColorCodes(String message) {
        char colorChar = '§';
        Matcher matcher = HEX_PATTERN.matcher(message);
        StringBuffer buffer = new StringBuffer(message.length() + 32);
        while (matcher.find()) {
            String group = matcher.group(1);
            matcher.appendReplacement(buffer, colorChar + "x" + colorChar + group
                    .charAt(0) + colorChar + group.charAt(1) + colorChar + group
                    .charAt(2) + colorChar + group.charAt(3) + colorChar + group
                    .charAt(4) + colorChar + group.charAt(5));
        }
        return matcher.appendTail(buffer).toString();
    }
}
