package com.loganmagnan.hubcore.scoreboard;

import com.loganmagnan.hubcore.HubCore;
import com.loganmagnan.hubcore.playerdata.PlayerData;
import com.loganmagnan.hubcore.playerdata.PlayerSettings;
import com.loganmagnan.hubcore.utils.BungeeUtils;
import com.loganmagnan.hubcore.utils.ColorUtils;
import com.loganmagnan.hubcore.utils.Utils;
import com.loganmagnan.vortex.api.VortexAPI;
import org.bukkit.Bukkit;
import org.bukkit.ChatColor;
import org.bukkit.entity.Player;
import org.bukkit.scheduler.BukkitRunnable;
import org.bukkit.scoreboard.DisplaySlot;
import org.bukkit.scoreboard.Objective;
import org.bukkit.scoreboard.Scoreboard;
import org.bukkit.scoreboard.Team;

import java.util.ArrayList;
import java.util.List;
import java.util.regex.Matcher;
import java.util.regex.Pattern;

public class SimpleBoard {

    private HubCore main = HubCore.getInstance();

    private static final int MAX_LINES = 15;
    // private static final int MAX_PREFIX_LENGTH = 16; - Minecraft now uses Components for this, so we don't need to worry about it
    private static final String[] BLANKS = new String[MAX_LINES];

    private Pattern HEX_PATTERN = Pattern.compile("&#([A-Fa-f0-9]{6})");

    static {
        for (int index = 0; index < MAX_LINES; index++) {
            BLANKS[index] = new String(new char[]{ChatColor.COLOR_CHAR, (char) ('s' + index)});
        }
    }

    private final Team[] teams = new Team[MAX_LINES];
    private final Objective objective;

    public SimpleBoard() {
        this(Bukkit.getScoreboardManager().getNewScoreboard());
    }

    public SimpleBoard(Scoreboard board) {
        board.clearSlot(DisplaySlot.SIDEBAR);

        objective = board.registerNewObjective("sidebar", "dummy");
        objective.setDisplaySlot(DisplaySlot.SIDEBAR);

        for (int index = 0; index < MAX_LINES; index++) {
            teams[index] = board.registerNewTeam(BLANKS[index]);
            teams[index].addEntry(BLANKS[index]);
        }
    }

    public List<String> getSpawnScoreboard(Player player, PlayerData playerData) {
        List<String> lines = new ArrayList<>();

        String rank = "";

        if (this.main.getServer().getPluginManager().getPlugin("Vault") == null) {
            if (this.main.getServer().getPluginManager().getPlugin("Vortex") == null) {
                rank = "None";
            } else {
                rank = VortexAPI.getRank(player).getPrefix();
            }
        } else {
            rank = this.main.getChat().getGroupPrefix(player.getWorld(), this.main.getChat().getPrimaryGroup(player));
        }

        for (String string : this.main.getScoreboardConfig().getConfig().getStringList("SCOREBOARD.LINES.SPAWN")) {
            lines.add(
                    string
                            .replace("%line%", Utils.scoreboardBar)
                            .replace("%player%", player.getName())
                            .replace("%online%", String.valueOf(BungeeUtils.getPlayerCount(null)))
                            .replace("%rank%", colorize(translateHexColorCodes(rank)))
                            .replace("%skyblockOnline%", String.valueOf(BungeeUtils.getPlayerCount("SkyBlock")))
                            .replace("%prisonOnline%", String.valueOf(BungeeUtils.getPlayerCount("Prison")))
                            .replace("%survivalOnline%", String.valueOf(BungeeUtils.getPlayerCount("Survival")))
            );
        }

        return lines;
    }

    public String colorize(String message) {
        return ChatColor.translateAlternateColorCodes('&', message);
    }

    public String translateHexColorCodes(String message) {
        char colorChar = '§';
        Matcher matcher = HEX_PATTERN.matcher(message);
        StringBuffer buffer = new StringBuffer(message.length() + 32);
        while (matcher.find()) {
            String group = matcher.group(1);
            matcher.appendReplacement(buffer, colorChar + "x" + colorChar + group
                    .charAt(0) + colorChar + group.charAt(1) + colorChar + group
                    .charAt(2) + colorChar + group.charAt(3) + colorChar + group
                    .charAt(4) + colorChar + group.charAt(5));
        }
        return matcher.appendTail(buffer).toString();
    }

    public void setTitle(String title) {
        objective.setDisplayName(title);
    }

    public void setLine(int index, String line) {
        if (index < 0 || index >= MAX_LINES) {
            throw new IndexOutOfBoundsException("Index must be between 0 and " + MAX_LINES);
        }

        if (line == null) {
            line = "";
        }

        line = ColorUtils.getMessageType(line);

        /*
         if (line.length() > MAX_PREFIX_LENGTH) {
             line = line.substring(0, MAX_PREFIX_LENGTH);
         }
         */
        teams[index].setPrefix(line);
        objective.getScore(BLANKS[index]).setScore(MAX_LINES - index);
    }

    public void removeLine(int index) {
        if (index < 0 || index >= MAX_LINES) {
            throw new IndexOutOfBoundsException("Index must be between 0 and " + MAX_LINES);
        }

        teams[index].setPrefix(BLANKS[index]);
        objective.getScoreboard().resetScores(BLANKS[index]);
    }

    public void clear() {
        for (int index = 0; index < MAX_LINES; index++) {
            teams[index].setPrefix(BLANKS[index]);
            objective.getScoreboard().resetScores(BLANKS[index]);
        }
    }

    public void displayList(List<String> lines) {
        // we can't let this flicker
        if (lines.size() > MAX_LINES) {
            throw new IllegalArgumentException("List cannot be larger than " + MAX_LINES);
        }

        for (int index = 0; index < MAX_LINES; index++) {
            if (index < lines.size()) {
                setLine(index, lines.get(index));
            } else {
                removeLine(index);
            }
        }
    }

    public void display(Player player) {
        player.setScoreboard(objective.getScoreboard());
    }

    public void hide(Player player) {
        player.setScoreboard(Bukkit.getScoreboardManager().getMainScoreboard());
    }

    public void dispose() {
        objective.unregister();
    }
}